import unittest
from ..compose_generator import ComposeGenerator


class TestComposeServices(unittest.TestCase):
    compose_with_no_services = """
version: 3.9
volumes:
  db-data:
    driver: flocker
    driver_opts:
      size: "10GiB"
"""

    compose_with_one_service = """
services:
  frontend:
    image: awesome/webapp
"""

    compose_with_one_service_complex = """
services:
  frontend:
    image: awesome/webapp
    container_name: myfrontend
    cpu_count: 1.5
"""

    compose_with_two_services = """
services:
  frontend:
    image: awesome/webapp
  backend:
    image: awesome/backend
"""

    def test_empty_services_throws_error(self):
        with self.assertRaises(KeyError):
            ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_no_services)

    def test_name_matches_with_one_service(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_one_service)
        self.assertIsNotNone(compose_file.services["frontend"])

    def test_compose_path_matches_name_with_one_service(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_one_service)
        self.assertEqual(compose_file.services["frontend"].compose_path, 'services/frontend')

    def test_names_match_with_two_services(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_two_services)
        self.assertEqual(list(compose_file.services.keys()), ["frontend", "backend"])

    def test_compose_paths_matches_name_with_two_services(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_two_services)
        self.assertEqual(compose_file.services["frontend"].compose_path, 'services/frontend')
        self.assertEqual(compose_file.services["backend"].compose_path, 'services/backend')

    def test_container_name_assigned_from_service(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_one_service_complex)
        self.assertEqual(compose_file.services["frontend"].container_name, "myfrontend")

    def test_cpu_count_assigned_from_service(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_one_service_complex)
        self.assertEqual(compose_file.services["frontend"].cpu_count, 1.5)

    def test_expose_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        self.assertIs(type(compose_file.services["frontend"].expose[0]), int)
        self.assertIn(3000, compose_file.services["frontend"].expose)

    def test_multiple_expose_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_multiple_expose()
        self.assertEqual(compose_file.services["frontend"].expose, [3000, 4000, 5000])

    def test_command_string_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        self.assertEqual(compose_file.services["frontend"].command.command_string(), "bundle exec thin -p 3000")

    def test_command_list_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_command_list()
        self.assertEqual("bundle exec thin -p 3000", compose_file.services["frontend"].command.command_string())

    def test_command_list_with_quotes_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_command_list_with_quotes()
        self.assertEqual('echo "hello world"', compose_file.services["frontend"].command.command_string())

    def test_structured_ports_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_structured_ports()
        self.assertEqual(80, compose_file.services["frontend"].ports[0].target)
        self.assertEqual(8443, compose_file.services["frontend"].ports[1].published)
        self.assertEqual("192.168.1.11", compose_file.services["frontend"].ports[1].host_ip)

    def test_entrypoint_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_entrypoint_no_command()
        self.assertEqual("/code/entrypoint.sh", compose_file.services["frontend"].entrypoint.command_string())

    def test_entrypoint_list_from_service(self):
        entrypoint = "php -d zend_extension=/usr/local/lib/php/extensions/no-debug-non-zts-20100525/xdebug.so"
        entrypoint += " -d memory_limit=-1 vendor/bin/phpunit"
        compose_file = ComposeGenerator.get_compose_with_entrypoint_as_list_no_command()
        self.assertEqual(entrypoint, compose_file.services["frontend"].entrypoint.command_string())

    def test_entrypoint_and_command_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_entrypoint_and_command()
        self.assertEqual('/code/entrypoint.sh echo "hello world"', compose_file.services["frontend"].entrypoint_and_command())

    def test_mem_reservation_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_memory_reservation()
        self.assertEqual(1610612736, compose_file.services["frontend"].mem_reservation.value)
        self.assertEqual(1.5, compose_file.services["frontend"].mem_reservation.as_gigabytes())


if __name__ == '__main__':
    unittest.main()



import unittest
import os
from ..compose_generator import ComposeGenerator


class TestBracesNoUnderscoreNoDigitVariableInterpolation(unittest.TestCase):

    def test_uppercase_in_string_value(self):
        env_var = "TESTNAME"
        os.environ[env_var] = "fred"
        braced_env_var = "{" + env_var + "}"
        compose_file = ComposeGenerator.get_compose_with_string_value(braced_env_var)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/fred")

    def test_uppercase_with_default_when_unset_in_string_value(self):
        env_var = "DEFAULTUNSET"
        os.unsetenv(env_var)
        braced_env_with_default_unset = "{" + env_var + ":-bob}"
        compose_file = ComposeGenerator.get_compose_with_string_value(braced_env_with_default_unset)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/bob")

    def test_uppercase_in_decimal_value(self):
        env_var = "TESTCPUCOUNT"
        os.environ[env_var] = "1.5"
        braced_env_var = "{" + env_var + "}"
        compose_file = ComposeGenerator.get_compose_with_decimal_value(braced_env_var)
        self.assertEqual(compose_file.services["frontend"].cpu_count, 1.5)

    def test_uppercase_two_variables_in_string_value(self):
        first_env_var = "HOSTPORT"
        second_env_var = "CONTAINERPORT"
        os.environ[first_env_var] = "8080"
        os.environ[second_env_var] = "80"
        braced_first_env_var = "{" + first_env_var + "}"
        braced_second_env_var = "{" + second_env_var + "}"
        compose_file = ComposeGenerator.get_with_two_environment_variables_in_string_value(braced_first_env_var, braced_second_env_var)
        self.assertEqual(f"{compose_file.services['frontend'].ports[0]}", "8080:80/tcp")
        self.assertEqual(compose_file.services['frontend'].ports[0].published, "8080")
        self.assertEqual(compose_file.services['frontend'].ports[0].target, "80")

    def test_lowercase_in_string_value(self):
        env_var = "testname"
        os.environ[env_var] = "fred"
        braced_env_var = "{" + env_var + "}"
        compose_file = ComposeGenerator.get_compose_with_string_value(braced_env_var)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/fred")

    def test_lowercase_with_default_when_unset_in_string_value(self):
        env_var = "defaultunset"
        os.unsetenv(env_var)
        braced_env_with_default_unset = "{" + env_var + ":-bob}"
        compose_file = ComposeGenerator.get_compose_with_string_value(braced_env_with_default_unset)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/bob")

    def test_lowercase_in_decimal_value(self):
        env_var = "testcpucount"
        os.environ[env_var] = "1.5"
        braced_env_var = "{" + env_var + "}"
        compose_file = ComposeGenerator.get_compose_with_decimal_value(braced_env_var)
        self.assertEqual(compose_file.services["frontend"].cpu_count, 1.5)

    def test_lowercase_two_variables_in_string_value(self):
        first_env_var = "hostport"
        second_env_var = "containerport"
        os.environ[first_env_var] = "8080"
        os.environ[second_env_var] = "80"
        braced_first_env_var = "{" + first_env_var + "}"
        braced_second_env_var = "{" + second_env_var + "}"
        compose_file = ComposeGenerator.get_with_two_environment_variables_in_string_value(braced_first_env_var, braced_second_env_var)
        self.assertEqual(f"{compose_file.services['frontend'].ports[0]}", "8080:80/tcp")
        self.assertEqual(compose_file.services["frontend"].ports[0].published, "8080")
        self.assertEqual(compose_file.services["frontend"].ports[0].target, "80")


if __name__ == '__main__':
    unittest.main()



import unittest
from ..compose_generator import ComposeGenerator


class TestServiceCredentialSpec(unittest.TestCase):

    def test_service_credential_spec_with_file_path(self):
        compose_file = ComposeGenerator.get_compose_with_file_credential_spec()
        self.assertEqual(compose_file.services["frontend"].credential_spec.file, "my-credential-spec.json")

    def test_service_credential_spec_with_registry_path(self):
        compose_file = ComposeGenerator.get_compose_with_registry_credential_spec()
        self.assertEqual(compose_file.services["frontend"].credential_spec.registry, "my-credential-spec")

    def test_service_credential_spec_with_config_path(self):
        compose_file = ComposeGenerator.get_compose_with_config_credential_spec()
        self.assertEqual(compose_file.services["frontend"].credential_spec.config, "my_credential_spec")



import re
from pycomposefile.compose_element import ComposeStringOrListElement


class Command(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = str
        super().__init__(config, key, compose_path)

    def command_string(self):
        capture = re.compile(r"\w+(\s\w+)+")
        string = ""
        for v in self:
            if len(self) > 1 and capture.match(v):
                string += f"\"{v}\""
            else:
                string += f"{v} "
        return string.lstrip().rstrip()



import unittest

from ..compose_generator import ComposeGenerator


class TestServiceOrdering(unittest.TestCase):

    @unittest.skip("unexpected failure when run as group.")
    def test_service_order(self):
        compose_file = ComposeGenerator.get_compose_with_service_dependencies()
        self.assertEqual(list(compose_file.ordered_services.keys()), ['db', 'redis', 'frontend', 'reporting'])

    def test_multiple_service_order(self):
        compose_file = ComposeGenerator.get_compose_with_multiple_service_dependencies()
        self.assertEqual(list(compose_file.ordered_services.keys()), ['redis', 'db', 'frontend', 'reporting'])

    def test_circular_service_order(self):
        with self.assertRaises(RecursionError):
            ComposeGenerator.get_compose_with_circular_service_dependencies()

    def test_service_order_with_conditions(self):
        compose_file = ComposeGenerator.get_compose_with_service_dependencies_and_conditions()
        self.assertEqual(compose_file.services["frontend"].depends_on[0].condition, 'service_healthy')


if __name__ == '__main__':
    unittest.main()



from pycomposefile.compose_element import ComposeElement


class CredentialSpec(ComposeElement):
    element_keys = {
        "file": (str,
                 "https://github.com/compose-spec/compose-spec/blob/master/spec.md#credential_spec"),
        "registry": (str,
                     "https://github.com/compose-spec/compose-spec/blob/master/spec.md#credential_spec"),
        "config": (str,
                   "https://github.com/compose-spec/compose-spec/blob/master/spec.md#credential_spec")
    }



import re

from decimal import Decimal
from pycomposefile.compose_element import ComposeElement, ComposeListOrMapElement, ComposeByteValue


class ResourceDetails(ComposeElement):
    element_keys = {
        "cpus": (Decimal, ""),
        "memory": (ComposeByteValue, ""),
        "pids": (int,
                 "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#pids"),
        "devices": (None,
                    "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#devices"),
    }


class Resources(ComposeElement):
    element_keys = {
        "limits": (ResourceDetails.from_parsed_yaml, ""),
        "reservations": (ResourceDetails.from_parsed_yaml, ""),
    }


class PlacementSpec(ComposeListOrMapElement):
    pass


class Placement(ComposeElement):
    element_keys = {
        "constraints": (PlacementSpec, "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#constraints"),
        "preferences": (PlacementSpec, "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#preferences"),
    }


class Labels(ComposeListOrMapElement):
    pass


class DeployTimespan(str):

    @classmethod
    def from_parsed_str(cls, value):
        timespan_validator = re.compile(r"\d+(ns|us|ms|s|m|h)")
        if not timespan_validator.match(value):
            # Should this default like it is currently or should it raise an error?
            value = "0s"
        return cls(value)


class UpdateConfig(ComposeElement):
    element_keys = {
        "parallelism": (int, ""),
        "delay": (DeployTimespan.from_parsed_str, ""),
        "failure_action": ((str, ["continue", "pause", "rollback"]), ""),
        "monitor": (DeployTimespan.from_parsed_str, ""),
        # TODO: find an example, not sure what this value looks like
        #  https://github.com/compose-spec/compose-spec/blob/master/deploy.md#update_config
        "max_failure_ratio": (str, ""),
        "order": ((str, ["stop-first", "start-first"]), ""),
    }


class Deploy(ComposeElement):
    element_keys = {
        "endpoint_mode": ((str, ["vip", "dnsrr"]), ""),
        "labels": (Labels, ""),
        "mode": ((str, ["global", "replicated"]), ""),
        "replicas": (int, ""),
        "resources": (Resources.from_parsed_yaml, ""),
        "rollback_config": (UpdateConfig.from_parsed_yaml, ""),
        "update_config": (UpdateConfig.from_parsed_yaml, ""),
        "placement": (Placement.from_parsed_yaml,
                      "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#placement"),
        "restart_policy": (None,
                           "https://github.com/compose-spec/compose-spec/blob/master/deploy.md#restart_policy"),
    }



from pycomposefile.compose_element import ComposeStringOrListElement

CAP_LIST = [
    "CAP_AUDIT_CONTROL",
    "CAP_AUDIT_READ",
    "CAP_AUDIT_WRITE",
    "CAP_BLOCK_SUSPEND",
    "CAP_BPF",
    "CAP_CHECKPOINT_RESTORE",
    "CAP_SYS_ADMIN",
    "CAP_CHOWN",
    "CAP_DAC_OVERRIDE",
    "CAP_DAC_READ_SEARCH",
    "CAP_FOWNER",
    "CAP_DAC_READ_SEARCH",
    "CAP_FSETID",
    "CAP_IPC_LOCK",
    "CAP_IPC_OWNER",
    "CAP_KILL",
    "CAP_LEASE",
    "CAP_LINUX_IMMUTABLE",
    "CAP_MAC_ADMIN",
    "CAP_MAC_OVERRIDE",
    "CAP_MKNOD",
    "CAP_NET_ADMIN",
    "CAP_NET_BIND_SERVICE",
    "CAP_NET_BROADCAST",
    "CAP_NET_RAW",
    "CAP_PERFMON",
    "CAP_SYS_ADMIN",
    "CAP_SETGID",
    "CAP_SETFCAP",
    "CAP_SETPCAP",
    "CAP_SETUID",
    "CAP_SYS_ADMIN",
    "CAP_BPF",
    "CAP_SYS_BOOT",
    "CAP_SYS_CHROOT",
    "CAP_SYS_MODULE",
    "CAP_SYS_NICE",
    "CAP_SYS_PACCT",
    "CAP_SYS_PTRACE",
    "CAP_SYS_RAWIO",
    "CAP_SYS_RESOURCE",
    "CAP_SYS_TIME",
    "CAP_SYS_TTY_CONFIG",
    "CAP_SYSLOG",
    "CAP_WAKE_ALARM",
]


class Cap(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = str
        self.valid_values = CAP_LIST
        super().__init__(config, key, compose_path)



from .compose_datatype_transformer import ComposeDataTypeTransformer


class ComposeStringOrListElement(ComposeDataTypeTransformer, list):
    transform = None
    valid_values = None

    def __init__(self, config, key=None, compose_path=None,):
        if compose_path is not None:
            self.compose_path = f"{compose_path}/{key}"

        if isinstance(config, list):
            for v in config:
                if isinstance(v, str):
                    v = v.strip("[]").rstrip().lstrip().strip("'")
                    self.append_transform(v)
                else:
                    self.append_transform(v, key, compose_path)
        else:
            self.append_transform(config, key, compose_path)

    def append(self, __object) -> None:
        if isinstance(__object, list):
            for element in __object:
                super().append(element)
        else:
            super().append(__object)

    def append_transform(self, config_value, key=None, compose_path=None):
        if isinstance(config_value, str) or isinstance(config_value, int):
            self.append(self.transform_supported_data(self.transform, config_value, self.valid_values))
        else:
            self.append(self.transform(config_value, key, compose_path))



import unittest
import os
from ..compose_generator import ComposeGenerator


class TestNoBraceNoUnderscoreNoDigitEnvironmentVariable(unittest.TestCase):

    def test_uppercase_in_string_value(self):
        env_var = "TESTNAME"
        os.environ[env_var] = "fred"
        compose_file = ComposeGenerator.get_compose_with_string_value(env_var)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/fred")

    def test_uppercase_in_decimal_value(self):
        env_var = "TESTCPUCOUNT"
        os.environ[env_var] = "1.5"
        compose_file = ComposeGenerator.get_compose_with_decimal_value(env_var)
        self.assertEqual(compose_file.services["frontend"].cpu_count, 1.5)

    def test_uppercase_two_variables_in_string_value(self):
        first_env_var = "HOSTPORT"
        second_env_var = "CONTAINERPORT"
        os.environ[first_env_var] = "8080"
        os.environ[second_env_var] = "80"
        compose_file = ComposeGenerator.get_with_two_environment_variables_in_string_value(first_env_var, second_env_var)
        self.assertEqual(f"{compose_file.services['frontend'].ports[0]}", "8080:80/tcp")
        self.assertEqual(compose_file.services["frontend"].ports[0].published, "8080")
        self.assertEqual(compose_file.services["frontend"].ports[0].target, "80")

    def test_lowercase_in_string_value(self):
        env_var = "testname"
        os.environ[env_var] = "fred"
        compose_file = ComposeGenerator.get_compose_with_string_value(env_var)
        self.assertEqual(compose_file.services["frontend"].image, "awesome/fred")

    def test_lowercase_in_decimal_value(self):
        env_var = "testcpucount"
        os.environ[env_var] = "1.5"
        compose_file = ComposeGenerator.get_compose_with_decimal_value(env_var)
        self.assertEqual(compose_file.services["frontend"].cpu_count, 1.5)

    def test_lowercase_two_variables_in_string_value(self):
        first_env_var = "hostport"
        second_env_var = "containerport"
        os.environ[first_env_var] = "8080"
        os.environ[second_env_var] = "80"
        compose_file = ComposeGenerator.get_with_two_environment_variables_in_string_value(first_env_var, second_env_var)
        self.assertEqual(f"{compose_file.services['frontend'].ports[0]}", "8080:80/tcp")
        self.assertEqual(compose_file.services["frontend"].ports[0].published, "8080")
        self.assertEqual(compose_file.services["frontend"].ports[0].target, "80")


if __name__ == '__main__':
    unittest.main()



import re
import os


class ComposeDataTypeTransformer():
    def transform_supported_data(self, transform, value, valid_values=None):
        if valid_values is not None:
            value = self.transform_and_validate_supported_data(value, transform, valid_values)
        else:
            value = transform(self.replace_environment_variables(value))
        return value

    def transform_and_validate_supported_data(self, value, data_transformer, valid_values):
        # TODO: if the data is an integer or decimal, should there be a "between" check?
        transformed = data_transformer(self.replace_environment_variables(value))
        if isinstance(transformed, int) and valid_values[0] <= transformed <= valid_values[1]:
            return transformed
        elif transformed not in valid_values:
            # TODO: Should this return None or should this raise?
            return None
        else:
            return transformed

    def replace_environment_variables(self, value):
        value = str(value)
        value = self.replace_environment_variables_with_empty_unset(value)
        value = self.replace_environment_variables_with_unset(value)
        value = self.replace_environment_variables_with_braces(value)
        value = self.replace_environment_variables_without_braces(value)

        return value

    def replace_environment_variables_with_empty_unset(self, value):
        capture = re.compile(r"\$\{(?P<variablename>\w+)\:-(?P<defaultvalue>\w+)\}")
        matches = capture.search(value)
        while matches:
            env_var = os.environ.get(matches.group("variablename"))
            default_value = matches.group("defaultvalue")
            if env_var is None or len(env_var) == 0:
                env_var = default_value
            value = re.sub(f"\\{matches[0]}", env_var, value)
            matches = capture.search(value)
        return value

    def replace_environment_variables_with_unset(self, value):
        capture = re.compile(r"\$\{(?P<variablename>\w+)-(?P<defaultvalue>\w+)\}")
        matches = capture.search(value)
        while matches:
            env_var = os.environ.get(matches.group("variablename"))
            default_value = matches.group("defaultvalue")
            if env_var is None:
                env_var = default_value
            value = re.sub(f"{matches[0]}", env_var, value)
            matches = capture.search(value)
        return value

    def replace_environment_variables_with_braces(self, value):
        capture = re.compile(r"\$\{(?P<variablename>\w+)\}")
        matches = capture.search(value)
        while matches:
            env_var = os.environ.get(matches.group("variablename"))
            value = re.sub(f"\\{matches[0]}", env_var, value)
            matches = capture.search(value)
        return value

    def replace_environment_variables_without_braces(self, value):
        capture = re.compile(r"\$(?P<variablename>\w+)")
        matches = capture.search(value)
        while matches:
            env_var = os.environ.get(matches.group("variablename"))
            value = re.sub(f"\\{matches[0]}", env_var, value)
            matches = capture.search(value)
        return value



import unittest
from ..compose_generator import ComposeGenerator


class TestComposeServices(unittest.TestCase):

    def test_blkio_from_service(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        self.assertEqual(compose_file.services["frontend"].blkio_config.weight, 300)
        self.assertEqual(compose_file.services["frontend"].blkio_config.weight_device[0].weight, 400)
        self.assertEqual(compose_file.services["frontend"].blkio_config.device_read_bps[0].path, "/dev/sdb")
        self.assertEqual(compose_file.services["frontend"].blkio_config.device_write_iops[0].rate, "30")


if __name__ == '__main__':
    unittest.main()




from pycomposefile import ComposeFile
import yaml


class ComposeGenerator:

    @staticmethod
    def convert_yaml_to_compose_file(yaml_string):
        compose_yaml = yaml.load(yaml_string, Loader=yaml.Loader)
        return ComposeFile(compose_yaml)

    @staticmethod
    def get_compose_with_string_value(variable_name):
        compose = """
services:
  frontend:
    image: awesome/${replace_me}
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose.format(replace_me=variable_name))

    @staticmethod
    def get_compose_with_decimal_value(variable_name):
        compose = """
services:
  frontend:
    image: awesome/website
    cpu_count: ${replace_me}
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose.format(replace_me=variable_name))

    @staticmethod
    def get_with_two_environment_variables_in_string_value(first_variable_name, second_variable_name):
        compose = """
services:
  frontend:
    image: awesome/website
    cpu_count: 1.5
    ports: "${replace_first}:${replace_second}"
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose.format(replace_first=first_variable_name, replace_second=second_variable_name))

    @staticmethod
    def get_compose_with_one_service_with_deploy():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "8080:80"
    expose: "3000"
    command: bundle exec thin -p 3000
    blkio_config:
      weight: 300
      weight_device:
        - path: /dev/sda
          weight: 400
      device_read_bps:
        - path: /dev/sdb
          rate: '12mb'
      device_read_iops:
        - path: /dev/sdb
          rate: 120
      device_write_bps:
        - path: /dev/sdb
          rate: '1024k'
      device_write_iops:
        - path: /dev/sdb
          rate: 30
    deploy:
      mode: replicated
      replicas: 2
      endpoint_mode: vip
      placement:
        constraints:
          - disktype=ssd
        preferences:
          datacenter=eastus
      resources:
        limits:
          cpus: '0.50'
          memory: 50M
          pids: 1
        reservations:
          cpus: '0.25'
          memory: 20M
      labels:
        com.example.description: "This label will appear on the web service"
        com.example.otherstuff: "random things"
      rollback_config:
        order: stop-first
        monitor: 5m
      update_config:
        order: start-first
        failure_action: rollback
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_one_service_with_multiple_expose():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "8080:80"
    expose:
      - 3000
      - "4000"
      - '5000'
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_command_list():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "8080:80"
    expose: "3000"
    command:
      - bundle
      - exec
      - thin
      - -p
      - 3000
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_command_list_with_quotes():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "8080:80"
    expose: "3000"
    command:
      - echo
      - "hello world"
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_structured_ports():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - target: 80
        host_ip: 127.0.0.1
        published: 8080
        protocol: tcp
        mode: host

      - target: 443
        host_ip: 192.168.1.11
        published: 8443
        protocol: tcp
        mode: host
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_structured_configs():
        compose = """
services:
  frontend:
    image: awesome/webapp
    configs:
      - source: my_config
        target: /redis_config
        uid: "103"
        gid: "103"
        mode: 0440

      - source: another_config
        target: /db_config
        uid: "105"
        gid: "105"
        mode: 0777
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_string_configs():
        compose = """
services:
  frontend:
    image: awesome/webapp
    configs:
      - source: my_config
      - source: another_config
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_entrypoint_no_command():
        compose = """
services:
  frontend:
    image: awesome/webapp
    entrypoint: /code/entrypoint.sh
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_entrypoint_as_list_no_command():
        compose = """
services:
  frontend:
    image: awesome/webapp
    entrypoint:
      - php
      - -d
      - zend_extension=/usr/local/lib/php/extensions/no-debug-non-zts-20100525/xdebug.so
      - -d
      - memory_limit=-1
      - vendor/bin/phpunit
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_entrypoint_and_command():
        compose = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "8080:80"
    expose: "3000"
    entrypoint: /code/entrypoint.sh
    command:
      - echo
      - "hello world"
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_file_credential_spec():
        compose = """
services:
  frontend:
    image: awesome/webapp
    credential_spec:
      file: my-credential-spec.json
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_registry_credential_spec():
        compose = """
services:
  frontend:
    image: awesome/webapp
    credential_spec:
      registry: my-credential-spec
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_config_credential_spec():
        compose = """
services:
  frontend:
    image: awesome/webapp
    credential_spec:
      config: my_credential_spec
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_service_dependencies():
        compose = """
services:
  frontend:
    image: awesome/webapp
    depends_on:
      - db
      - redis
  redis:
    image: redis
  db:
    image: postgres
  reporting:
    image: reporting
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_multiple_service_dependencies():
        compose = """
services:
  frontend:
    image: awesome/webapp
    depends_on:
      - db
      - redis
  db:
    image: postgres
    depends_on:
      - redis
  redis:
    image: redis
  reporting:
    image: reporting
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_circular_service_dependencies():
        compose = """
services:
  frontend:
    image: awesome/webapp
    depends_on:
      - db
      - redis
  db:
    image: postgres
    depends_on:
      - redis
  redis:
    image: redis
    depends_on:
      - reporting
  reporting:
    image: reporting
    depends_on:
      - db
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_service_dependencies_and_conditions():
        compose = """
services:
  frontend:
    image: awesome/webapp
    depends_on:
      db:
        condition: service_healthy
      redis:
        condition: service_started
  redis:
    image: redis
  db:
    image: postgres
  reporting:
    image: reporting
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)

    @staticmethod
    def get_compose_with_memory_reservation():
        compose = """
services:
  frontend:
    image: awesome/webapp
    mem_reservation: 1.5gb
"""
        return ComposeGenerator.convert_yaml_to_compose_file(compose)



from collections import OrderedDict
from pycomposefile.service import Service


class ComposeFile:
    ordered_services = OrderedDict()

    def __init__(self, deserialized_compose_file):
        version = deserialized_compose_file.pop("version", None)
        if version is not None:
            version = str(version)
        self.version = version
        self.services = OrderedDict()
        self.compose_path = "services"
        for service in deserialized_compose_file["services"].keys():
            self.services[service] = Service.from_parsed_yaml(
                deserialized_compose_file["services"][service],
                service,
                self.compose_path
            )
        for service_name in self.services.keys():
            self._append_ordered_service(service_name)

    def _append_ordered_service(self, service_name):
        service = self.services[service_name]
        if service.depends_on is None:
            self.ordered_services[service_name] = service
        else:
            for required_service in service.depends_on:
                if required_service not in self.ordered_services.keys():
                    self._append_ordered_service(required_service)
            self.ordered_services[service_name] = service



import unittest
from ..compose_generator import ComposeGenerator


class TestComposeVersion(unittest.TestCase):
    compose_with_no_version = """
services:
  frontend:
    image: awesome/webapp
    ports:
      - "443:8043"
    networks:
      - front-tier
      - back-tier
    configs:
      - httpd-config
    secrets:
      - server-certificate
"""

    compose_with_version_2 = """
version: 2.7
services:
  frontend:
    image: awesome/webapp
    ports:
      - "443:8043"
    networks:
      - front-tier
      - back-tier
    configs:
      - httpd-config
    secrets:
      - server-certificate
"""

    compose_with_version_3 = """
version: 3.8
services:
  frontend:
    image: awesome/webapp
    ports:
      - "443:8043"
    networks:
      - front-tier
      - back-tier
    configs:
      - httpd-config
    secrets:
      - server-certificate
"""

    compose_with_unknown_version = """
version: 100.1.alpha
services:
  frontend:
    image: awesome/webapp
    ports:
      - "443:8043"
    networks:
      - front-tier
      - back-tier
    configs:
      - httpd-config
    secrets:
      - server-certificate
"""

    def test_no_version_present(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_no_version)
        self.assertIsNone(compose_file.version)

    def test_v2_version_present(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_version_2)
        self.assertEqual(compose_file.version, "2.7")

    def test_v3_version_present(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_version_3)
        self.assertEqual(compose_file.version, "3.8")

    def test_unknown_version_present(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_unknown_version)
        self.assertEqual(compose_file.version, "100.1.alpha")


if __name__ == '__main__':
    unittest.main()



from .compose_datatype_transformer import ComposeDataTypeTransformer


class ComposeListOrMapElement(ComposeDataTypeTransformer, dict):
    def __init__(self, config, key=None, compose_path=None,):
        if compose_path is not None:
            self.compose_path = f"{compose_path}/{key}"

        if config is None:
            pass
        elif isinstance(config, dict):
            for key in config.keys():
                value = self.replace_environment_variables(config[key])
                key = self.replace_environment_variables(key)
                self[key] = value
        elif isinstance(config, list):
            for line in config:
                line = self.replace_environment_variables(line)
                key, value = line.split("=")
                self[key] = value
        elif isinstance(config, str):
            config = self.replace_environment_variables(config)
            key, value = config.split("=")
            self[key] = value

    # def __init__(self, config, key=None, compose_path=None,):
    #     if isinstance(config, dict):
    #         for key in config.keys():
    #             value = self.replace_environment_variables(config[key])
    #             key = self.replace_environment_variables(key)
    #             self[key] = value
    #     elif isinstance(config, list):
    #         pass
    #     elif isinstance(config, str):
    #         pass
    #     else:
    #         pass



from decimal import Decimal
from .compose_datatype_transformer import ComposeDataTypeTransformer


class ComposeByteValue(ComposeDataTypeTransformer):
    compose_path = ""
    value = 0.0
    _conversions = {
        'gb': 1073741824,
        'g': 1073741824,
        'mb': 1048576,
        'kb': 1024,
        'm': 1048576,
        'k': 1024,
        'b': 1,
    }

    def convert_value(self, value_string):
        value_string = value_string.lower()
        for key in self._conversions.keys():
            if value_string.endswith(key):
                return Decimal(value_string.rstrip(key)) * self._conversions[key]
        return Decimal(value_string)

    def __init__(self, config, compose_path=None):
        if compose_path is not None:
            self.compose_path = compose_path
        self.value = self.convert_value(config)

    def as_gigabytes(self):
        return self.value / self._conversions['gb']


class ComposeElement(ComposeDataTypeTransformer):
    element_keys = {}

    def __init__(self, config, compose_path=""):
        self.compose_path = compose_path
        for key in self.element_keys.keys():
            config_element = config.pop(key, None)
            key_config = self.element_keys[key]
            self.set_supported_property_from_config(key, key_config, config_element, compose_path)
        for key in config.keys():
            # raise Exception(f"Failed to map {key} in {compose_path}")
            pass

    def set_supported_property_from_config(self, key, key_config, value, compose_path):
        valid_values = None
        if type(key_config[0]) is tuple:
            transform, valid_values = key_config[0]
        else:
            transform = key_config[0]

        if transform is not None:
            if isinstance(value, dict):
                value = transform(value, key, compose_path)
            elif isinstance(value, list):
                value = transform(value, key, compose_path)
            elif value is not None:
                value = self.transform_supported_data(transform, value, valid_values)
        else:
            # TODO: Logging message if value was not None
            value = None
        self.__setattr__(key, value)

    @classmethod
    def from_parsed_yaml(cls, config, name, compose_path):
        if config is None:
            return None
        compose_path = f"{compose_path}/{name}"
        return cls(config, compose_path)



import unittest
from ..compose_generator import ComposeGenerator


class TestServiceConfig(unittest.TestCase):

    def test_service_config_from_string(self):
        compose_file = ComposeGenerator.get_compose_with_string_configs()
        self.assertEqual(compose_file.services["frontend"].configs[0].source, 'my_config')

    def test_service_config_from_dict(self):
        compose_file = ComposeGenerator.get_compose_with_structured_configs()
        self.assertEqual(compose_file.services["frontend"].configs[1].source, 'another_config')
        self.assertEqual(compose_file.services["frontend"].configs[1].target, '/db_config')



from pycomposefile.compose_element import ComposeElement, ComposeStringOrListElement


class Config(ComposeElement):
    element_keys = {
        "source": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
        "target": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
        "uid": (int, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
        "gid": (int, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
        "mode": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
    }

    def __init__(self, config_definition, key=None, compose_path=""):
        if isinstance(config_definition, str):
            config_definition = {
                "source": config_definition
            }
        super().__init__(config_definition, compose_path)


class Configs(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = Config
        super().__init__(config, key, compose_path)



import re
from pycomposefile.compose_element import ComposeElement, ComposeStringOrListElement


class Port(ComposeElement):
    element_keys = {
        "target": (int, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
        "host_ip": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
        "published": (int, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
        "protocol": ((str, ['tcp', 'udp']), "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
        "mode": ((str, ['host', 'ingress']), "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
    }

    def __init__(self, port_definition, key=None, compose_path=""):
        if isinstance(port_definition, str):
            self.parse_string(port_definition)
        else:
            super().__init__(port_definition, compose_path)

    def parse_string(self, port_definition):
        port_matcher = re.compile(r'(?P<HostIp>((\d+\.\d+\.\d+\.\d+:)|))(?P<HostPort>(\d+:)|)(?P<ContainerPort>\d+)(?P<Protocol>((/(tcp|udp))|))')
        matches = port_matcher.search(port_definition)
        host_ip = matches.group('HostIp')
        if host_ip == '':
            host_ip = None
        else:
            host_ip = host_ip.rstrip(":")
        host_port = matches.group('HostPort')
        if host_port == '':
            host_port = None
        else:
            host_port = host_port.rstrip(":")
        container_port = matches.group('ContainerPort')
        protocol = matches.group('Protocol')
        if protocol == '':
            protocol = 'tcp'

        self.__setattr__('host_ip', host_ip)
        self.__setattr__('published', host_port)
        self.__setattr__('target', container_port)
        self.__setattr__('protocol', protocol)
        self.__setattr__('mode', 'host')

    def __str__(self) -> str:
        port = ''
        if self.host_ip is not None:
            port += f"{self.host_ip}:"
        if self.published is not None:
            port += f"{self.published}:"
        port += f"{self.target}/{self.protocol}"
        return port


class Ports(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = Port
        super().__init__(config, 'ports', compose_path)



import setuptools
import os

long_desc = open("README.md").read()
required = ['pyyaml']  # Comma seperated dependent libraries name
version = os.environ.get("BUILD_TAG", "0.0.1a1").lstrip("v")

setuptools.setup(
    name="pycomposefile",
    version=version,
    author="Steven Murawski",
    author_email="steven.murawski@microsoft.com",
    license="MIT",
    description="Structured deserialization of Docker Compose files.",
    long_description=long_desc,
    long_description_content_type="text/markdown",
    url="https://github.com/smurawski/pycomposefile",
    packages=setuptools.find_packages(),
    key_words="docker compose",
    install_requires=required,
    python_requires=">=3.6",
)



from decimal import Decimal

from pycomposefile.service.service_blkio_config import BlkioConfig
from pycomposefile.service.service_deploy import Deploy
from pycomposefile.service.service_credential_spec import CredentialSpec
from pycomposefile.service.service_cap import Cap
from pycomposefile.service.service_configs import Configs
from pycomposefile.service.service_command import Command
from pycomposefile.service.service_ports import Ports
from pycomposefile.compose_element import ComposeElement, ComposeStringOrListElement, ComposeByteValue


class Expose(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = int
        super().__init__(config, key, compose_path)


class CpuSets(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = str
        super().__init__(config, key, compose_path)


class Dependency(str):
    def __new__(cls, config, key=None, compose_path=None, ) -> None:
        condition = None
        if isinstance(config, Dependency):
            return config
        if isinstance(config, tuple):
            name, detail = config
            condition = detail["condition"]
        else:
            name = config
        ob = super(Dependency, cls).__new__(cls, name)
        ob.__setattr__('condition', condition)
        return ob


class DependsOn(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = Dependency
        if isinstance(config, dict):
            config_to_list = []
            for key in config.keys():
                config_to_list.append((key, config[key]))
            config = config_to_list
        super().__init__(config, key, compose_path)


class Service(ComposeElement):
    element_keys = {
        "image": (str, ""),
        "container_name": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#container_name"),
        "cpu_count": (Decimal, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_count"),
        "entrypoint": (Command, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#entrypoint"),
        "command": (Command, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#command"),
        "deploy": (Deploy.from_parsed_yaml, "https://github.com/compose-spec/compose-spec/blob/master/deploy.md"),
        "expose": (Expose, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#expose"),
        "ports": (Ports, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#long-syntax-2"),
        "cpus": (Decimal, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpus"),
        "credential_spec": (CredentialSpec.from_parsed_yaml, ""),
        "blkio_config": (BlkioConfig.from_parsed_yaml,
                         "https://github.com/compose-spec/compose-spec/blob/master/spec.md#blkio_config"),
        "cpu_percent": (Decimal,
                        "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_percent"),
        "cpu_shares": (int,
                       "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_shares"),
        "cpu_period": (str,
                       "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_period"),
        "cpu_quota": (int,
                      "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_quota"),
        "cpu_rt_runtime": (str,
                           "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_rt_runtime"),
        "cpu_rt_period": (str,
                          "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpu_rt_period"),
        "cpuset": (list,
                   "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cpuset"),
        "build": (None,
                  "https://github.com/compose-spec/compose-spec/blob/master/build.md"),
        "cap_add": (Cap,
                    "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cap_add"),
        "cap_drop": (Cap,
                     "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cap_add"),
        "cgroup_parent": (str,
                          "https://github.com/compose-spec/compose-spec/blob/master/spec.md#cgroup_parent"),
        "configs": (Configs,
                    "https://github.com/compose-spec/compose-spec/blob/master/spec.md#configs"),
        "depends_on": (DependsOn, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#depends_on"),
        "mem_reservation": (ComposeByteValue, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#mem_reservation"),
    }

    def entrypoint_and_command(self):
        if self.command is None and self.entrypoint is None:
            return None
        else:
            container_entrypoint_and_command = ""
            if self.entrypoint is not None:
                container_entrypoint_and_command += self.entrypoint.command_string()
                container_entrypoint_and_command += " "
            if self.command is not None:
                container_entrypoint_and_command += self.command.command_string()
            return container_entrypoint_and_command




from pycomposefile.compose_element import ComposeElement, ComposeStringOrListElement


class IoConfig(ComposeElement):
    element_keys = {
        "path": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_bps-device_write_bps"),
        "rate": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_bps-device_write_bps"),
    }


class IoConfigList(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = IoConfig.from_parsed_yaml
        super().__init__(config, key, compose_path)


class WeightDevice(ComposeElement):
    element_keys = {
        "path": (str, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#weight_device"),
        "weight": ((int, [10, 1000]), "https://github.com/compose-spec/compose-spec/blob/master/spec.md#weight_device"),
    }


class WeightDeviceList(ComposeStringOrListElement):
    def __init__(self, config, key=None, compose_path=None):
        self.transform = WeightDevice.from_parsed_yaml
        super().__init__(config, key, compose_path)


class BlkioConfig(ComposeElement):
    element_keys = {
        "device_read_bps": (IoConfigList, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_bps-device_write_bps"),
        "device_write_bps": (IoConfigList, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_bps-device_write_bps"),
        "device_read_iops": (IoConfigList, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_iops-device_write_iops"),
        "device_write_iops": (IoConfigList, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#device_read_iops-device_write_iops"),
        "weight": ((int, [10, 1000]), "https://github.com/compose-spec/compose-spec/blob/master/spec.md#weight"),
        "weight_device": (WeightDeviceList, "https://github.com/compose-spec/compose-spec/blob/master/spec.md#weight_device"),
    }



import unittest
from ..compose_generator import ComposeGenerator


class TestComposeServiceDeploy(unittest.TestCase):
    compose_with_no_deploy = """
services:
  frontend:
    image: awesome/webapp
"""

    def test_service_with_no_deploy(self):
        compose_file = ComposeGenerator.convert_yaml_to_compose_file(self.compose_with_no_deploy)
        self.assertIsNone(compose_file.services["frontend"].deploy)

    def test_service_with_deploy(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertIsNotNone(deploy)
        self.assertEqual(deploy.endpoint_mode, "vip")

    def test_service_with_deploy_labels(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        expected = {
            "com.example.description": "This label will appear on the web service",
            "com.example.otherstuff": "random things"
        }
        self.assertDictEqual(deploy.labels, expected)

    def test_service_with_deploy_resources(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertIsNotNone(deploy)
        self.assertEqual(deploy.resources.limits.cpus, 0.50)

    def test_compose_path_for_service_with_deploy(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertEqual(deploy.compose_path, "services/frontend/deploy")

    def test_service_with_deploy_update_config(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertIsNotNone(deploy.update_config)

    def test_service_with_deploy_rollback_config(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertIsNotNone(deploy.rollback_config)
        self.assertEqual(deploy.rollback_config.monitor, "5m")

    def test_service_with_deploy_placement_preferences(self):
        compose_file = ComposeGenerator.get_compose_with_one_service_with_deploy()
        deploy = compose_file.services["frontend"].deploy
        self.assertEqual(deploy.placement.preferences["datacenter"], 'eastus')
        self.assertEqual(deploy.placement.constraints["disktype"], 'ssd')


if __name__ == '__main__':
    unittest.main()
